require("mapshmem.jl")

@everywhere function create_matrix(n::Integer,p::Float64)

	#generate nearest-neighbor laplacian, with periodic boundary conditions
	k=4*eye(n)-circshift(eye(n),1)-circshift(eye(n),-1)-circshift(eye(n),2)-circshift(eye(n),-2)


	#insert off-diagonal elements
	pr=p
	k=k-pr.*circshift(eye(n),-int(n/2))+pr.*eye(n)

	return k

end

#Trivial parallelism
function p_feidler(n,pp)

	lam = @parallel (hcat) for p =1:length(pp)
		k=4*eye(n)-circshift(eye(n),1)-circshift(eye(n),-1)-circshift(eye(n),2)-circshift(eye(n),-2)
	#insert off-diagonal elements
		pr=pp[p]
		k=k-pr.*circshift(eye(n),-int(n/2))+pr.*eye(n)
		real(eig(k)[1][2])
	end

	return pp,lam

end

function feidler(n,pp)

	lam=zeros(Float64,(length(pp)))

	for p =1:length(pp)
		k=4*eye(n)-circshift(eye(n),1)-circshift(eye(n),-1)-circshift(eye(n),2)-circshift(eye(n),-2)
		#insert off-diagonal elements
		pr=pp[p]
		k=k-pr.*circshift(eye(n),-int(n/2))+pr.*eye(n)
		lam[p]=real(eig(k)[1][2])
	end

	return pp,lam

end


#Shared memory
function sm_feidler(n,pp)

map_shmem(ShmemVar(:lam, Float64, length(pp)), "pfx")

@sync begin
	@parallel for p =1:length(pp)
		k=4*eye(n)-circshift(eye(n),1)-circshift(eye(n),-1)-circshift(eye(n),2)-circshift(eye(n),-2)
	#insert off-diagonal elements
		pr=pp[p]
		k=k-pr.*circshift(eye(n),-int(n/2))+pr.*eye(n)
		global lam; lam[p]=real(eig(k)[1][2])
	end
end

lamm=remotecall_fetch(1, () -> lam)
#writedlm("fiedler.txt", remotecall_fetch(1, () -> lam))
free_shmem(:lam, "pfx")
return pp, lam

end





